# synchro

synchro lets you execute a set of asynchronous functions in sequence or to wait until all of them
have finished executing.

## Installation

```
npm install synchro
```

## Example Code

This example creates two chains -- the first is executed in order, and the second one, out of order but
waited on.

``` js
var synchro = require('synchro');

// creates a synchronous chain, same as s1 = synchro.create(async = false)
var s1 = synchro.create();

s1
  .add(function (step) {
    console.log('starting first step...');
    setTimeout(function () {
      console.log('first step DONE after 2 secs.');
      step.next();
    }, 2000);
  })
  .add(function (step) {
    console.log('starting second step...');
    setTimeout(function () {
      console.log('second step DONE after 3 secs.');
      step.next();
    }, 3000);
  })
  .add(function (step) {
    console.log('starting third step...');
    setTimeout(function () {
      console.log('third step DONE after 3 secs.');
      step.next();
    }, 3000);
  });

// get notified when the chain is done
s1.on('done', function () {
  console.log('all steps executed in order!');
});

s1.run();

// creates an asynchronouse chain, same as s2 = synchro.create(async = true)
var s2 = synchro.create(true);

s2
  .add(function (step) {
    setTimeout(function () {
      step.next();
      console.log('callback 1 done...');
    }, Math.floor(Math.random() * 5000));
    console.log('waiting for callback 1...');
  })
  .add(function (step) {
    setTimeout(function () {
      console.log('callback 2 done...');
      step.next();
    }, Math.floor(Math.random() * 5000));
    console.log('waiting for callback 2...');
  })
  .add(function (step) {
    setTimeout(function () {
      console.log('callback 3 done...');
      step.next();
    }, Math.floor(Math.random() * 5000));
    console.log('waiting for callback 3...');
  });

// get notified when all tasks in the chain are done
s2.on('done', function () {
  console.log('all callbacks executed (may not be in order)!');
});

// run the chain
s2.run();
```

Running this code will output something like

```
starting first step...
waiting for callback 1...
waiting for callback 2...
waiting for callback 3...
callback 2 done...
first step DONE after 2 secs.
starting second step...
callback 3 done...
all callbacks executed (may not be in order)!
callback 1 done...
second step DONE after 3 secs.
starting third step...
third step DONE after 3 secs.
all steps executed in order!
```

Note that the order for callbacks 1, 2 and 3 are not guaranteed, but you can pass a function to run once they've
all been executed.

## License

(The MIT License)

Copyright (c) 2011 Jesus A. Domingo <jesus.domingo@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the 'Software'), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
